﻿using System;
using System.IO;
using System.Net;
using System.Reflection;
using System.Text;
using Microsoft.Office.Interop.Word;
using Microsoft.ReportingServices.Interfaces;
using Microsoft.ReportingServices.OnDemandReportRendering;
using System.Configuration;
using System.Diagnostics;

namespace CustomWordRenderingExtension
{
    public class CustomWordRenderer : IExtension, IRenderingExtension
    {

        #region IExtension Members
        public string LocalizedName
        {
            get
            {
                return "CUSTOM WORD";
            }
        }

        public void SetConfiguration(string configuration)
        {

        }

        #endregion

        #region IRenderingExtension Members

        public void GetRenderingResource(CreateAndRegisterStream createAndRegisterStreamCallback, System.Collections.Specialized.NameValueCollection deviceInfo)
        {

        }

        public bool IsProcessOpen(string name)
        {
            foreach (Process clsProcess in Process.GetProcesses())
            {

                if (clsProcess.ProcessName.ToLower().Contains(name))
                {
                    return true;
                }
            }
            return false;
        }

        public bool Render(Microsoft.ReportingServices.OnDemandReportRendering.Report report, System.Collections.Specialized.NameValueCollection reportServerParameters, System.Collections.Specialized.NameValueCollection deviceInfo, System.Collections.Specialized.NameValueCollection clientCapabilities, ref System.Collections.Hashtable renderProperties, CreateAndRegisterStream createAndRegisterStream)
        {
            string source = "Custom Rendering Extension";
            string message = string.Empty;
            string userName = string.Empty;

            try
            {
                // Get the rerport URL
                string downloadUrl = string.Concat(
                                            report.GetReportUrl(false),
                                            "&rs:Format=WORD&rs:Command=Render&rc:FileExtension=doc");

                // Concatenate parameters to report URL
                foreach (ReportParameter parameter in report.Parameters)
                {
                    if (parameter.MultiValue)
                    {
                        for (int i = 0; i < parameter.Instance.Values.Count; i++)
                        {
                            downloadUrl = string.Concat(downloadUrl, "&", parameter.Instance.Prompt, "=", parameter.Instance.Values[i].ToString());
                        }
                    }
                    else
                    {
                        if (!string.IsNullOrEmpty(parameter.Instance.Prompt))
                        {
                            downloadUrl = string.Concat(downloadUrl, "&", parameter.Instance.Prompt, "=", parameter.Instance.Value);
                        }
                    }
                }

                // Get File Paths from device info configuration
                object sourceHeaderfileName = deviceInfo["SourceHeaderFileName"].ToString();
                string bodyFileName = Path.GetFileNameWithoutExtension(deviceInfo["SourceBodyFileName"].ToString());
                string timeStamp = DateTime.Now.ToFileTime().ToString();
                bodyFileName += timeStamp;
                string bodyFileExtension = Path.GetExtension(deviceInfo["SourceBodyFileName"].ToString());
                string bodyFilePath = Path.GetDirectoryName(deviceInfo["SourceBodyFileName"].ToString());
                object sourceBodyfileName = string.Format("{0}\\{1}{2}", bodyFilePath, bodyFileName, bodyFileExtension);

                string mergedFileName = Path.GetFileNameWithoutExtension(deviceInfo["MergedFileName"].ToString());
                mergedFileName += timeStamp;
                string mergedFilePath = Path.GetDirectoryName(deviceInfo["MergedFileName"].ToString());
                object mergedFile = string.Format("{0}\\{1}{2}", mergedFilePath, mergedFileName, bodyFileExtension);
                EventLog.WriteEntry(source, mergedFile.ToString(), EventLogEntryType.Information);

                byte[] doc;

                // Download the report rendered in word format
                WebClient webClient = new WebClient();
                webClient.UseDefaultCredentials = true;
                doc = webClient.DownloadData(downloadUrl);

                // Write the downloaded document to disc
                FileStream fs = new FileStream(sourceBodyfileName.ToString(), FileMode.Create);
                BinaryWriter bw1 = new BinaryWriter(fs);
                bw1.Write(doc);
                bw1.Flush();
                bw1.Close();
                fs.Close();

                // Initialize objects for word automation
                object missing = Missing.Value;
                object readOnlyFalse = false;
                object readOnlyTrue = true;
                object isVisible = false;
                object saveChanges = true;
                object oTrue = true;
                object styleHeading1 = Microsoft.Office.Interop.Word.WdBuiltinStyle.wdStyleHeading1;
                object styleHeading2 = Microsoft.Office.Interop.Word.WdBuiltinStyle.wdStyleHeading2;



                // Application Object
                Microsoft.Office.Interop.Word.Application wordApp = null;

                // Document Objects
                Microsoft.Office.Interop.Word.Document sourceWordDocumentHeader;
                Microsoft.Office.Interop.Word.Document sourceWordDocumentBody;
                Microsoft.Office.Interop.Word.Document mergedWordDocument;

                // Launch the word Application
                wordApp = new Microsoft.Office.Interop.Word.Application();

                // Hide the User Interface
                wordApp.Visible = false;

                // Open the Main Body Document
                sourceWordDocumentBody = wordApp.Documents.Open(ref sourceBodyfileName, ref missing, ref readOnlyFalse, ref missing, ref missing, ref missing,
                                                       ref missing, ref missing, ref missing, ref missing, ref missing, ref isVisible,
                                                       ref missing, ref missing, ref missing, ref missing);

                // Open the header document as read only
                sourceWordDocumentHeader = wordApp.Documents.Open(ref sourceHeaderfileName, ref missing, ref readOnlyTrue, ref missing, ref missing, ref missing,
                                                       ref missing, ref missing, ref missing, ref missing, ref missing, ref isVisible,
                                                       ref missing, ref missing, ref missing, ref missing);

                // Create a new document to store the merged output
                mergedWordDocument = wordApp.Documents.Add(ref missing, ref missing, ref missing, ref missing);

                // Merge the header and body contents and paste in the merged document
                Object range = mergedWordDocument.Range(ref missing, ref missing);
                Section headerSection = mergedWordDocument.Sections.First;
                headerSection.PageSetup.Orientation = WdOrientation.wdOrientPortrait;
                Range sourceHeaderRange = sourceWordDocumentHeader.Range(ref missing, ref missing);
                sourceHeaderRange.Copy();
                headerSection.Range.Paste();
                Section bodySection = mergedWordDocument.Sections.Add(ref missing, ref missing);
                bodySection.PageSetup.Orientation = WdOrientation.wdOrientLandscape;
                Range sourcebodyRange = sourceWordDocumentBody.Range(ref missing, ref missing);
                sourcebodyRange.Copy();
                bodySection.Range.Paste();

                // Save merged document
                mergedWordDocument.SaveAs(ref mergedFile, ref missing, ref missing, ref missing, ref missing, ref missing,
                    ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing);


                // Convert Table Entries to Proper Heading for TOC
                foreach (Microsoft.Office.Interop.Word.Field myField in mergedWordDocument.Fields)
                {
                    Microsoft.Office.Interop.Word.Range rngFieldCode = myField.Code;
                    String fieldText = rngFieldCode.Text;

                    if (fieldText.StartsWith(" TC"))
                    {
                        Int32 end = fieldText.IndexOf("\"", 5);
                        Int32 fieldNameLength = end - 5;
                        String fieldName = fieldText.Substring(5, fieldNameLength);
                        rngFieldCode.set_Style(ref styleHeading1);

                        // Clear the TC tag
                        myField.Copy();
                        myField.Delete();
                    }
                }

                // Update TOC
                mergedWordDocument.TablesOfContents[1].Update();

                // Save Merged Document
                mergedWordDocument.SaveAs(ref mergedFile, ref missing, ref missing, ref missing, ref missing, ref missing,
                    ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing, ref missing);

                // Close document and application
                mergedWordDocument.Close(ref missing, ref missing, ref missing);
                sourceWordDocumentHeader.Close(ref missing, ref missing, ref missing);
                sourceWordDocumentBody.Close(ref missing, ref missing, ref missing);
                wordApp.Quit(ref missing, ref missing, ref missing);

                // Stream the document
                Stream s = createAndRegisterStream(
                           "Monthly Website Usage Report",
                           "doc",
                           Encoding.Default,
                           "application/msword",
                           true,
                           StreamOper.CreateAndRegister);

                BinaryWriter bw = new BinaryWriter(s);
                byte[] byteArray = File.ReadAllBytes(mergedFile.ToString());
                bw.Write(byteArray);
                bw.Flush();

                return true;
            }
            catch (Exception ex)
            {
                message = ex.ToString();

                // Write exception to event log
                EventLog.WriteEntry(source, message, EventLogEntryType.Error);

                // Report rendering failed
                return false;
            }
        }

        public bool RenderStream(string streamName, Microsoft.ReportingServices.OnDemandReportRendering.Report report, System.Collections.Specialized.NameValueCollection reportServerParameters, System.Collections.Specialized.NameValueCollection deviceInfo, System.Collections.Specialized.NameValueCollection clientCapabilities, ref System.Collections.Hashtable renderProperties, CreateAndRegisterStream createAndRegisterStream)
        {
            return true;
        }

        #endregion
    }
}

